function [w,dopt,Jmds] = mdsteq(h,Lw,dmin,dmax,LorQ);
% [w,dopt,Jmds] = mdsteq(h,Lw,dmin,dmax,LorQ);
%
% Minimum Delay Spread (MDS) TEQ design.
% This computes the TEQ which minimizes the RMS delay spread
% of the effective channel.
%
% w = optimal MDS TEQ impulse response, optimized over delay
% dopt = optimal delay (optimal in the MDS sense)
%    Note that delays start at zero, so the appropriate
%    index into the effective channel vector is dopt+1.
% Jmds = vector of achieved delay spreads
% h = channel impulse response
% Lw = TEQ length
% [dmin:dmax] = range of delay values to check.
%    dmin should be zero or greater.
%    dmax should be at most (length(h)+Lw-2).
% LorQ is a flag indicating linear or quadratic weights.
%    LorQ = 1 --> quadratic weights, as in ref. [1] below.
%    LorQ = 0 --> linear weights, as in ref. [2] below.
%    The original MDS algorithm uses quadratic weights.
%    With linear weights, the cost is not really delay spread.
% 
% The algorithm is from:
%
% [1] R. Schur and J. Speidel, "An Efficient Equalization Method to
% Minimize Delay Spread in OFDM/DMT Systems," Proc. IEEE Int. Conf.
% on Comm., pp. 1481--1485, Helsinki, Finland, June 2001.
%
% [2] A. Tkacenko and P. P. Vaidyanathan, "Noise Optimized
% Eigenfilter Design of Time-Domain Equalizers For DMT Systems,"
% Proc. IEEE Int. Conf. on Comm., New York, NY, April--May 2002.
%
% Copyright 2003, Cornell University
%
% This program (mdsteq.m) is free software; you can redistribute it
% and/or modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; either version 2 of
% the License, or (at your option) any later version.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
% 02111-1307 USA; or you can find it on the World Wide Web at 
% http://www.fsf.org.
%
% Programmer: Richard K. Martin
% Version: May 15, 2003
% 
% The author is with the Broadband Adaptive Receiver Design project
% at the School of Electrical and Computer Engineering, Cornell
% University, Ithaca, NY.  He can be reached at:
% frodo@ece.cornell.edu
% http://bard.ece.cornell.edu/

% define filter lengths
Lh = length(h);  % channel length
Lc = Lh+Lw-1;    % length of effective channel

% error checks:
% force h into a column vector,
% force 0 <= dmin <= Lc-1,
% force dmin <= dmax <= Lc-1.
h = h(:);
Dmin = 0;
Dmax = Lc-1;
dmin = min(max(dmin,Dmin),Dmax);
dmax = max(min(dmax,Dmax),dmin);

Nd = dmax-dmin+1;       % number of delay values
ds = [dmin:dmax];       % range of delay values
Jmds = inf*ones(Nd,1);  % initialize cost function vector
Jtemp = inf;            % initialize optimum cost
w = zeros(Lw,1);        % initialize optimum TEQ
dopt = dmin;            % initialize optimum delay

H = convmtx(h,Lw);      % channel convolution matrix, c = H*w
U = chol(H'*H);         % Cholesky decomposition, (H'*H) = U'*U
Ui = pinv(U);           % H is full col. rank, so inverse exists
HUi = H*Ui;

for i = 1:Nd  % loop through delay values
   d = ds(i);           % current delay value

   % compute MDS TEQ and squared delay spread for this delay
   if LorQ % quadratic weights
      Q = diag( ([0:Lc-1]-d).^2 );
   else % linear weights
      Q = diag( abs([0:Lc-1]-d) );
   end % if LorQ
   W = HUi' * Q * HUi;
   [evecs,evals] = eig(W,'nobalance');
   [Jmds(i),ind] = min(diag(evals));
   v = evecs(:,ind);    % TEQ in alternate basis

   % Check if the solution for this delay is better
   % than the previous best solution (in the MDS sense).
   % If it is, keep it.
   if Jmds(i) < Jtemp
      Jtemp = Jmds(i);  % revise best cost acheived
      dopt = d;         % revise best delay
      w = Ui*v;         % revise optimum TEQ
   end % if Jmds(i) < Jtemp
   
end % for i = 1:Nd

% compute delay spreads from squares of delay spreads
Jmds = sqrt(Jmds);
